use bevy::prelude::*;
use bevy_rand::{prelude::WyRand, resource::GlobalEntropy};
use rand::Rng;

use crate::METER;

#[derive(Debug, Event)]
pub struct SpawnEvent {
    pos: (f32, f32),
    spawner: Entity,
}

#[derive(Component)]
pub struct Spawner {
    pos: (f32, f32),
    size: (f32, f32),
    spawn_cooldown: Timer,
    cur_enemies: u8,
    max_enemies: u8,
    amount_range: std::ops::Range<u8>,
}

pub fn enemies_plugin(app: &mut App) {
    app.add_event::<SpawnEvent>()
        .add_systems(Startup, setup_spawners)
        .add_systems(Update, (spawn_enemies, do_spawn));
}

pub fn setup_spawners(mut commands: Commands) {
    commands.spawn(Spawner {
        pos: (-6.5 * METER, 5.5 * METER),
        size: (14. * METER, 1.5 * METER),
        spawn_cooldown: Timer::from_seconds(5., TimerMode::Repeating),
        cur_enemies: 0,
        max_enemies: 10,
        amount_range: 2..4,
    });
}

#[derive(Component)]
pub struct Enemy;

// TODO: waves n stuff
// and gamestate lolz
pub fn spawn_enemies(
    mut ev_spawn: EventWriter<SpawnEvent>,
    mut rng: ResMut<GlobalEntropy<WyRand>>,
    mut query: Query<(Entity, &mut Spawner)>,
    time: Res<Time>,
) {
    for (entity, mut spawner) in query.iter_mut() {
        if spawner.spawn_cooldown.tick(time.delta()).finished()
            && spawner.cur_enemies <= spawner.max_enemies
        {
            // TODO: randomness
            (0..rng.gen_range(spawner.amount_range.clone())).for_each(|_| {
                if spawner.cur_enemies < spawner.max_enemies {
                    spawner.cur_enemies += 1;
                    ev_spawn.send(SpawnEvent {
                        pos: (
                            rng.gen_range(
                                (spawner.pos.0 + 0.5 * METER)
                                    ..((spawner.pos.0) + (spawner.size.0 - 0.5 * METER)),
                            ),
                            rng.gen_range(
                                (spawner.pos.1 + 0.5 * METER)
                                    ..((spawner.pos.1) + (spawner.size.1 - 0.5 * METER)),
                            ),
                        ),
                        spawner: entity,
                    });
                }
            })
        }
    }
}

pub fn do_spawn(
    mut ev_spawn: EventReader<SpawnEvent>,
    mut commands: Commands,
    asset_server: Res<AssetServer>,
) {
    for ev in ev_spawn.read() {
        println!("spawn!: {} {}", ev.pos.0 / METER, ev.pos.1 / METER);
        commands.spawn((
            Enemy,
            SpriteBundle {
                texture: asset_server.load("sprites/enemy.png"),
                transform: Transform::from_xyz(ev.pos.0, ev.pos.1, 1.),
                ..default()
            },
        ));
    }
}
