use bevy::prelude::*;
use bevy_rand::{plugin::EntropyPlugin, prelude::WyRand};
use bevy_rapier2d::prelude::*;
use enemy::enemies_plugin;
use player::{add_player, move_player};
use rand::SeedableRng;
use some_bevy_tools::{camera_2d::Camera2DPlugin, controller_2d::SimpleTopDownControllerPlugin};
const METER: f32 = 64.;

mod enemy;
mod player;
mod scene;

fn main() {
    let r = rand::rngs::SmallRng::from_seed(*b"Meow UwU nyaaa~ mrrrp~ meew miau");
    App::new()
        .insert_resource(RapierConfiguration {
            gravity: Vec2::new(0.0, 0.0),
            ..Default::default()
        })
        .add_plugins(EntropyPlugin::<WyRand>::default())
        .add_plugins(enemies_plugin)
        .add_plugins(DefaultPlugins)
        .add_plugins(RapierPhysicsPlugin::<NoUserData>::pixels_per_meter(METER))
        .add_plugins(RapierDebugRenderPlugin::default())
        .add_plugins(Camera2DPlugin)
        .add_plugins(SimpleTopDownControllerPlugin)
        .add_systems(Startup, (scene::setup_scene, add_player))
        .add_systems(Update, move_player)
        .run();
}
